/*
 * COPYRIGHT (c) 2010-2015 by Jan Vrany, Jan Kurs and Marcel Hlopko
 * COPYRIGHT (c) 2014-2015 by Tomas Heger
 *                            SWING Research Group, Czech Technical University in Prague
 *
 * This software is furnished under a license and may be used
 * only in accordance with the terms of that license and with the
 * inclusion of the above copyright notice. This software may not
 * be provided or otherwise made available to, or used by, any
 * other person. No title to or ownership of the software is
 * hereby transferred.
 */
package stx.libjava.groovy;

import java.util.Iterator;
import java.util.Set;
import java.util.HashSet;

import groovy.lang.Binding;

import org.codehaus.groovy.ast.ClassNode;
import org.codehaus.groovy.ast.ImportNode;
import org.codehaus.groovy.classgen.GeneratorContext;
import org.codehaus.groovy.control.CompilationFailedException;
import org.codehaus.groovy.control.CompilerConfiguration;
import org.codehaus.groovy.control.SourceUnit;
import org.codehaus.groovy.control.customizers.ImportCustomizer;


/**
 * A customised GroovyShell that remembers all imports across multiple
 * evaluation. 
 * 
 * @author Jan Vrany <jan.vrany [at] fit.cvut.cz>
 *
 */
@stx.libjava.annotation.Package("stx:libjava")
public class GroovyShell extends groovy.lang.GroovyShell {
    
    protected AutoImporter importer;
   
    public GroovyShell() {
        this(null, new Binding());
    }

    public GroovyShell(Binding binding) {
        this(null, binding);
    }

    public GroovyShell(CompilerConfiguration config) {
        this(new Binding(), config);
    }

    public GroovyShell(Binding binding, CompilerConfiguration config) {
        this(null, binding, config);
    }

    public GroovyShell(ClassLoader parent, Binding binding) {
        this(parent, binding, CompilerConfiguration.DEFAULT);
    }

    public GroovyShell(ClassLoader parent) {
        this(parent, new Binding(), CompilerConfiguration.DEFAULT);
    }
    
    public GroovyShell(ClassLoader parent, Binding binding, final CompilerConfiguration config) {        
        super(parent,binding,config);
        importer = new AutoImporter();
        config.addCompilationCustomizers(importer);              
    }
    
    /**
     * Imports a given class so it is ready to use in successive evaluations. 
     * 
     * @param className fully qualified class name to import
     */
    public void addClassImport(String className) {
        importer.addImports(className);
    }
    
    /**
     * Imports all classes in given package so they are ready to use 
     * in successive evaluations. 
     * 
     * @param className fully qualified package name to import
     */
    public void addPackageImport(String className) {
        importer.addStarImports(className);
    }

    
    /**
     * A an compiler customizer that search the code being evaluated and remembers
     * all imports so they are available in successively evaluated code. 
     *
     * @author Jan Vrany <jan.vrany [at] fit.cvut.cz>
     *
     */
    public static class AutoImporter extends ImportCustomizer {
        Set<String> classes = new HashSet<String>();
        Set<String> packages = new HashSet<String>();
        
        public void call(final SourceUnit source, final GeneratorContext context, final ClassNode classNode) throws CompilationFailedException {               
            super.call(source, context, classNode);
            
            for (Iterator<ImportNode> i = source.getAST().getImports().iterator(); i.hasNext();) {                            
                String name = i.next().getClassName();
                if ( ! classes.contains(name) ) {
                    classes.add(name);
                    this.addImports(name);                    
                }
            }
            for (Iterator<ImportNode> i = source.getAST().getStarImports().iterator(); i.hasNext();) {
                String name = i.next().getPackageName();
                if ( ! packages.contains(name) ) {
                    packages.add(name);
                    this.addStarImports(name);
                }
            }
        }
              
    }    
}
